/*
 * bresenham.c
 *
 *  Created on: Oct 2, 2019
 *      Author: michael.moretti
 */

#include <stdint.h>

// Everything is uint32_t/int32_t coz it's faster on 32-bit mcus

static inline void __attribute__((always_inline)) drawHorizontalLine(uint16_t *buf, uint32_t bufWidth, uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2, uint16_t color)
{
    uint32_t length;
    if (x1 > x2)
    {
        length = x1 - x2;
        buf += x2;
    }
    else
    {
        length = x2 - x1;
        buf += x1;
    }
    buf += y1 * bufWidth;

    for (uint32_t i = 0; i <= length; i++)
    {
        *buf++ = color;
    }
}

static inline __attribute__((always_inline)) void drawVerticalLine(uint16_t *buf, uint32_t bufWidth, uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2, uint16_t color)
{
    uint32_t length;
    if (y1 > y2)
    {
        length = y1 - y2;
        buf += y2 * bufWidth;
    }
    else
    {
        length = y2 - y1;
        buf += y1 * bufWidth;
    }
    buf += x1;

    for (uint32_t i = 0; i <= length; i++)
    {
        *buf = color;
        buf += bufWidth;
    }
}

__attribute__((section(".text.$SRAM_ITC")))
void mmDrawLineFaster(uint16_t *buf, uint32_t bufWidth, uint32_t x1, uint32_t y1, uint32_t x2, uint32_t y2, uint16_t color)
{
    // Shortcut horizontal and vertical lines
    if (x1 == x2)
    {
        return drawVerticalLine(buf, bufWidth, x1, y1, x2, y2, color);
    }
    else if (y1 == y2)
    {
        return drawHorizontalLine(buf, bufWidth, x1, y1, x2, y2, color);
    }

    uint32_t x,y;
    uint32_t dx, dy;
    int32_t err;

    if (x1 > x2)
        dx = x1 - x2;
    else
        dx = x2 - x1;

    if (y1 > y2)
        dy = y1 - y2;
    else
        dy = y2 - y1;

    if ( dy > dx )
    {
        if ( y1 > y2 )
        {
            uint32_t tmp = x1; x1 = x2; x2 = tmp;
            tmp = y1; y1 = y2; y2 = tmp;
        }

        err = dy >> 1;

        int32_t xstep;
        if (x2 > x1)
            xstep = 1;
        else
            xstep = -1;

        buf += x1 + y1 * bufWidth;
        for( y = y1; y <= y2; y++ )
        {
            *buf = color;
            err -= dx;
            if ( err < 0 )
            {
                buf += xstep;
                err += dy;
            }
            buf += bufWidth;
        }
    }
    else
    {
        if ( x1 > x2 )
        {
            uint32_t tmp = x1; x1 = x2; x2 = tmp;
            tmp = y1; y1 = y2; y2 = tmp;
        }

        err = dx >> 1;

        int32_t ystep;
        if (y2 > y1)
            ystep = bufWidth;
        else
            ystep = -bufWidth;

        buf += x1 + y1 * bufWidth;
        for( x = x1; x <= x2; x++ )
        {
            *buf++ = color;
            err -= dy;
            if ( err < 0 )
            {
                buf += ystep;
                err += dx;
            }
        }
    }
}
